#region Copyright Syncfusion Inc. 2001 - 2006
//
//  Copyright Syncfusion Inc. 2001 - 2006. All rights reserved.
//
//  Use of this code is subject to the terms of our license.
//  A copy of the current license can be obtained at any time by e-mailing
//  licensing@syncfusion.com. Re-distribution in any form is strictly
//  prohibited. Any infringement will be prosecuted under applicable laws. 
//
#endregion

#region file using directives
using System;
using System.Collections;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Windows.Forms;
using System.Diagnostics;
using System.Text;

using Syncfusion.XlsIO;
using Syncfusion.XlsIO.Implementation;
using Syncfusion.Windows.Forms.Grid;
using Syncfusion.Windows.Forms.Tools;
using Syncfusion.Windows.Forms.Chart;
using Syncfusion.Drawing;

#endregion

namespace Syncfusion.GridExcelConverter
{
  /// <summary>
  /// This controls provides support for Exporting datas from a gridControl into an Excel spreadsheet
  /// for verification and/or computation. This Control automatically Copies the Grid's Styles, Formats 
  /// to Excel. 
  /// The GridExcelConverter Control  is derived from 
  ///<see cref="GridExcelConverterBase"/>.
  /// </summary>
  [ToolboxItem(false)]
  public class GridExcelConverterControl : GridExcelConverterBase
  {
    #region Class members
    //    /// <summary>
    //    /// 
    //    /// </summary>
    //    private static readonly Hashtable s_hashBorderIndexToExcelLineStyle = new Hashtable();
    #endregion

    #region Class Initialize/Finalize methods
    /// <summary>
    /// Static constructor.
    /// </summary>
    static GridExcelConverterControl()
    {
      FormulaUtil.RegisterAdditionalAlias( "POW", ExcelFunction.POWER );
      FormulaUtil.RegisterAdditionalAlias( "AVG", ExcelFunction.AVERAGE );
    }
    //    /// <summary>
    //    /// Static constructor.
    //    /// </summary>
    //    static GridExcelConverterControl()
    //    {
    //      FillBordersHash();
    //    }
    /// <summary>
    /// Default constructor.
    /// </summary>
    public GridExcelConverterControl()
    {
    }

    /// <summary>
    /// Clean up any resources being used.
    /// </summary>
    protected override void Dispose( bool disposing )
    {
      base.Dispose( disposing );
    }

    #endregion

    #region Class Export Grid to Excel methods

      GridModel gm;
      public GridModel GridModel
      {
          get { return gm; }
      }
    /// <summary>
    /// Converts grid into excel file.
    /// </summary>
    /// <param name="grid">Grid to export.</param>
    /// <param name="fileName">File to export into.</param>
    public void GridToExcel( GridControl grid, string fileName )
    {

      GridToExcel( grid, fileName, ConverterOptions.Default );
    }
    /// <summary>
    /// Converts grid into excel file.
    /// </summary>
    /// <param name="grid">Grid to export.</param>
    /// <param name="fileName">File to export into.</param>
    /// <param name="options">Converter options.</param>
    public void GridToExcel( GridControl grid, string fileName, ConverterOptions options )
    {
      GridToExcel( grid.Model, fileName, options );
    }
    /// <summary>
    /// Converts grid into excel file.
    /// </summary>
    /// <param name="grid">Grid to export.</param>
    /// <param name="sheet">Destination worksheet.</param>
    public void GridToExcel( GridControl grid, IWorksheet sheet )
    {
      GridToExcel( grid, sheet, ConverterOptions.Default );
    }
    /// <summary>
    /// Converts grid into excel file.
    /// </summary>
    /// <param name="grid">Grid to export.</param>
    /// <param name="sheet">Destination worksheet.</param>
    /// <param name="options">Converter options.</param>
    public void GridToExcel( GridControl grid, IWorksheet sheet, ConverterOptions options )
    {
      GridToExcel( grid.Model, sheet, options );
    }
    /// <summary>
    /// Converts grid into excel file.
    /// </summary>
    /// <param name="grid">Grid to export.</param>
    /// <param name="fileName">File to export into.</param>
    public void GridToExcel( GridModel grid, string fileName )
    {
      GridToExcel( grid, fileName, ConverterOptions.Default );
    }
    /// <summary>
    /// Converts grid into excel file.
    /// </summary>
    /// <param name="grid">Grid to export.</param>
    /// <param name="fileName">File to export into.</param>
    /// <param name="options">Converter options.</param>
    public void GridToExcel( GridModel grid, string fileName, ConverterOptions options )
    {
      ExcelEngine engine = CreateEngine();
      IWorkbook book = engine.Excel.Workbooks.Create( 1 );
      IWorksheet sheet = book.Worksheets[ 0 ];

      GridToExcel( grid, sheet, options );

      book.Close( true, fileName );
    }
    /// <summary>
    /// Converts grid into excel worksheet.
    /// </summary>
    /// <param name="grid">Grid to export.</param>
    /// <param name="sheet">Destination worksheet.</param>
    public void GridToExcel( GridModel grid, IWorksheet sheet )
    {
      GridToExcel( grid, sheet, ConverterOptions.Default );
    }
    /// <summary>
    /// Converts grid into excel worksheet.
    /// </summary>
    /// <param name="grid">Grid to export.</param>
    /// <param name="sheet">Destination worksheet.</param>
    /// <param name="options">Convert options.</param>
    public void GridToExcel( GridModel grid, IWorksheet sheet, ConverterOptions options )
    {
      if( grid == null )
        throw new ArgumentNullException( "grid" );

      if( sheet == null )
        throw new ArgumentNullException( "sheet" );

      this.gm = grid;
      int iRowDelta = ( ( options & ConverterOptions.RowHeaders ) != 0 ) ? 1 : 0;
      int iColumnDelta = ( ( options & ConverterOptions.ColumnHeaders ) != 0 ) ? 1 : 0;

      CopyMergesFromGrid( grid, sheet, options );
      CopyRowHeightFromGrid( grid, sheet, 1 - iRowDelta );
      CopyColumnWidthFromGrid( grid, sheet, 1 - iColumnDelta );
      int iFrozenRows = grid.Rows.FrozenCount;
      int iFrozenCols = grid.Cols.FrozenCount;

      if( iFrozenCols > 0 && iFrozenCols > 0 )
      {
        sheet.Range[ iFrozenRows + 1, iFrozenCols + 1 ].FreezePanes();
      }

      int iMaxRow = Math.Min( WorksheetBaseImpl.DEF_MAX_ROW_ONE_INDEX,
        grid.RowCount + iRowDelta );

      int iMaxColumn = Math.Min( WorksheetBaseImpl.DEF_MAX_COLUMN_ONE_INDEX,
        grid.ColCount + iColumnDelta );

#if ( !ASPNET && MEASURE_PERFORMANCE )
      DateTime start = DateTime.Now;
#endif

      // TODO: Optimization is needed
      for( int iRow = 1; iRow <= iMaxRow; iRow++ )
      {
        for( int iColumn = 1; iColumn <= iMaxColumn; iColumn++ )
        {
          int iGridRow = iRow - iRowDelta;
          int iGridCol = iColumn - iColumnDelta;

          GridStyleInfo gridCell = grid[ iGridRow, iGridCol ];

          if( gridCell.IsEmpty ) continue;

          GridCellToExcel( grid, iGridRow, iGridCol, sheet.Range[ iRow, iColumn ] );
        }
      }

#if ( !ASPNET && MEASURE_PERFORMANCE )
      TimeSpan totalTime = DateTime.Now - start;
      Debug.WriteLine( string.Format( "Total time of iteration through cells is {0}", totalTime ) );
#endif
    }
    /// <summary>
    /// Copies merged ranges.
    /// </summary>
    /// <param name="grid">Source grid.</param>
    /// <param name="sheet">Destination worksheet.</param>
    /// <param name="options">Converter options.</param>
    private void CopyMergesFromGrid( GridModel grid, IWorksheet sheet, ConverterOptions options )
    {
      if( grid == null )
        throw new ArgumentNullException( "grid" );

      if( sheet == null )
        throw new ArgumentNullException( "sheet" );

      GridModelCoveredRanges coveredRanges = grid.CoveredRanges;
      GridRangeInfoList ranges = coveredRanges.Ranges;

      for( int i = 0, len = ranges.Count; i < len; i++ )
      {
        GridRangeInfo rangeInfo = ranges[ i ];
        Debug.WriteLine( i, "Range index" );

        string strMessage = string.Format( "({0}, {1})-({2}, {3})", rangeInfo.Top, rangeInfo.Left, rangeInfo.Bottom, rangeInfo.Right );
        Debug.WriteLine( strMessage, "Range rect" );

        if( rangeInfo.Top == 0 || rangeInfo.Left == 0 ) continue;

        sheet.Range[ rangeInfo.Top, rangeInfo.Left, rangeInfo.Bottom, rangeInfo.Right ].Merge();
      }
    }
    /// <summary>
    /// Copies one grid cell into excel cell.
    /// </summary>
    /// <param name="grid">Source grid.</param>
    /// <param name="iRow">Grid cell row index to copy.</param>
    /// <param name="iColumn">Grid cell column index to copy.</param>
    /// <param name="range">Destination excel range.</param>
    private void GridCellToExcel( GridModel grid, int iRow, int iColumn, IRange range )
    {
      if( grid == null )
        throw new ArgumentNullException( "grid" );

      if( range == null )
        throw new ArgumentNullException( "range" );

      GridStyleInfo gridCell = grid[ iRow, iColumn ];

      if( gridCell.IsEmpty ) return;

      switch( gridCell.CellType )
      {
        case DEF_CELL_TYPE_IMAGE:
          Image bmp = gridCell.CellValue as Image;

          if( bmp != null )
          {
            int iRangeRow = range.Row;
            int iRangeColumn = range.Column;

            IPictureShape picture = range.Worksheet.Pictures.AddPicture(
              iRangeRow, iRangeColumn, bmp );
          }
          break;

        case DEF_CELL_TYPE_COMBOBOX:
          range.Text = gridCell.FormattedText;
          break;

        case DEF_CELL_TYPE_PROGRESSBAR:
          GridProgressBarInfo progressBar = gridCell.ProgressBar;
          int iValue = progressBar.ProgressValue;

          if( progressBar.TextStyle == ProgressBarTextStyles.Percentage )
          {

            int iMin = progressBar.Minimum;
            int iMax = progressBar.Maximum;
            int iRange = iMax - iMin;
            int iPercent = Math.Min( 100, iValue * 100 / iRange );
            range.Number = iPercent / 100.0;
            range.NumberFormat = "0%";
          }
          else
          {
            range.Number = iValue;
          }
          break;

        case DEF_CELL_TYPE_RTF:
          range.Text = gridCell.FormattedText;
          break;

		  case DEF_CELL_TYPE_CTRL:
			  if(gridCell.Control is Syncfusion.Windows.Forms.Chart.ChartControl)
				EmbedChart(range, gridCell);
			  break;
        default:
          object objValue = gridCell.CellValue;
          
          if( objValue is DateTime )
          {
            range.DateTime = ( DateTime )objValue;
          }
          else if( objValue is string )
          {
            if( gridCell.CellType == DEF_CELL_TYPE_FORMULA && gridCell.Text[ 0 ] == '=' )
            {
              range.Formula = gridCell.Text;
            }
            else
            {
              range.Text = gridCell.Text;
            }
          }
          else
          {
            range.Value2 = objValue;
          }
          break;
      }

      CopyStyles( grid, iRow, iColumn, range );
    }
    #endregion

	  void EmbedChart(IRange range, GridStyleInfo gridCell)
	  {
		  
		  Syncfusion.Windows.Forms.Chart.ChartControl chartControl = gridCell.Control as Syncfusion.Windows.Forms.Chart.ChartControl;

		  IWorksheet sheet = range.Worksheet;
		  //Clustered Column Chart
		  IChartShape chart = sheet.Charts.Add();
		  //Set Chart Type
		  chart.ChartType = ExcelChartType.Column_Clustered;


		  //Chart Title
		  chart.ChartTitle = chartControl.Text;
			  
			
		  //X axis title
		  chart.PrimaryCategoryAxis.Title = chartControl.PrimaryXAxis.Title;

		  //Y axis title
		  chart.PrimaryValueAxis.Title = chartControl.PrimaryYAxis.Title;

          
		  //Format Chart Area
		  IChartFrameFormat chartArea = chart.ChartArea;

		  //Initializes a new series instance and adds it to the series collection of the chart.
		  IChartSerie serie = chart.Series.Add();
					
		  //Specify the chart type of the series.
		  serie.SerieType = ExcelChartType.Column_Clustered;
					

		  //Specify the name of the series. This will be displayed as the text of the legend.
		  ChartSeries chartSeries = (chartControl.Model.Series[0] as ChartSeries);
		  serie.Name = chartSeries.Text;

          
          GridRangeInfo gridValueRange = gridCell.Tag as GridRangeInfo;
          
		  string valueRange = GridRangeInfo.GetAlphaLabel(gridValueRange.Right) + gridValueRange.Top.ToString() + ":" +
              GridRangeInfo.GetAlphaLabel(gridValueRange.Right) + gridValueRange.Bottom.ToString();
		  string categoryRange = GridRangeInfo.GetAlphaLabel(gridValueRange.Left) + gridValueRange.Top.ToString() + ":" +
			  GridRangeInfo.GetAlphaLabel(gridValueRange.Left) + gridValueRange.Bottom.ToString();

//		  string dataRange = GridRangeInfo.GetAlphaLabel(gridValueRange.Left) + gridValueRange.Top.ToString() + ":" +
//			  GridRangeInfo.GetAlphaLabel(gridValueRange.Right) + gridValueRange.Bottom.ToString();
//
//		  chart.DataRange = sheet.Range[dataRange];
		  
        //Specify the value ranges for the series.
		  serie.Values  = sheet.Range[valueRange];
					

		  //Specify the Category labels for the series.
		  serie.CategoryLabels = sheet.Range[categoryRange];
          
          GridRangeInfo gridRange = GridModel.CoveredRanges.FindRange(gridCell.CellIdentity.RowIndex, gridCell.CellIdentity.ColIndex);
          //Embedded chart position.
          chart.TopRow = gridRange.Top;
          chart.BottomRow = gridRange.Bottom+1;
          chart.LeftColumn = gridRange.Left;
          chart.RightColumn = gridRange.Right+1;
					
	  }
  }
}
